import React, { ChangeEvent, Component, MouseEvent } from 'react';

type TodoAppProps = {};
type TodoAppState = {
  items: Array<{ name: string, completed: boolean }>,
  newItem: string
};

export class TodoApp extends Component<TodoAppProps, TodoAppState> {

  constructor(props: TodoAppProps) {
    super(props);
    this.state = { items: [], newItem: "" };
  }

  render = (): JSX.Element => {
    return (
      <div>
        <h1>To-Do List</h1>
        {this.renderItems()}
        <p>Check the item to mark it completed. Click delete to remove it.</p>
        <div>
          <label htmlFor='item-input'>New item: </label>
          <input type="text" value={this.state.newItem}
            onChange={this.doNewItemChange} id='item-input' />
          <button type="button"
            onClick={this.doAddClick}>Add Item</button>
        </div>
      </div>
    );
  }

  renderItems = (): JSX.Element => {
    if (this.state.items.length === 0) {
      return <p>No items. Add one to get started!</p>
    }

    const items = [];
    for (let i = 0; i < this.state.items.length; i++) {
      const item = this.state.items[i];
      items.push(
        <li key={i}>
          { /* Note: this uses a new type of input called "checkbox", which is a boolean input.
               Unlike "normal" input boxes, it uses checked, not value to store its state.

               Separately, we're passing in an anonymous function, the
               evt => this.doItemToggleComplete(evt, i)
               as the onChange parameter. This creates a new function that takes in one argument,
               the evt, and this function just calls this.doItemToggleComplete but with the extra
               index argument. This lets us have our event handlers have extra parameters!
            */ }
          <input type="checkbox" id={"check" + i} checked={item.completed}
            onChange={evt => this.doItemToggleComplete(evt, i)} />
          <label htmlFor={"check" + i}>
            {/* This ternary strikethroughs the name if the item is completed.
                <s> is them "strikethrough" element. */}
            {item.completed ? <s>{item.name}</s> : item.name}
          </label>{" "} {/* This adds an explicit space between the elements */}
          <button onClick={evt => this.doItemRemove(evt, i)}>Delete</button>
        </li>
      )
    }
    return <ul>{items}</ul>;
  };

  doNewItemChange = (evt: ChangeEvent<HTMLInputElement>) => {
    this.setState({ newItem: evt.target.value });
  }

  doAddClick = (_evt: MouseEvent<HTMLButtonElement>) => {
    // Ignore the request if the user hasn't entered a name.
    // (Would be better to disable the button in this case.)
    const name = this.state.newItem.trim();
    if (name.length == 0)
      return;

    const items = this.state.items.concat([{ name: name, completed: false }]);
    this.setState({ items: items, newItem: "" });
  }

  doItemToggleComplete = (_evt: ChangeEvent<HTMLInputElement>, index: number) => {
    const item = this.state.items[index];

    const items = this.state.items.slice(0, index)
      .concat([{ name: item.name, completed: !item.completed }]) // "flipping" the completed field.
      .concat(this.state.items.slice(index + 1));

    this.setState({ items: items });
  };

  doItemRemove = (_evt: MouseEvent<HTMLButtonElement>, index: number) => {
    const items = this.state.items.slice(0, index)
      .concat(this.state.items.slice(index + 1));

    this.setState({ items: items });
  }
}
